@@ -133,7 +133,7 @@ gdk_pixbuf__svg_image_load_increment (gpointer data,
         }
 
         if (!rsvg_handle_write (context->handle, buf, size, error)) {
-                g_clear_pointer(error, g_error_free);
+                g_clear_error(error);
                 rsvg_propagate_error (error, _("Error writing"), ERROR_WRITING);
                 return FALSE;
         }
