@@ -1,4 +1,4 @@
-use cairo::{self, MatrixTrait};
+use cairo;
 use markup5ever::local_name;
 
 use crate::bbox::BoundingBox;
