@@ -1,36 +1,43 @@
 use cssparser::{
-    self, parse_important, AtRuleParser, CowRcStr, DeclarationParser, Parser, ParserInput,
+    self,
+    parse_important,
+    AtRuleParser,
+    CowRcStr,
+    DeclarationParser,
+    Parser,
+    ParserInput,
 };
 use std::collections::hash_map::{Entry, Iter as HashMapIter};
 use std::collections::HashMap;
 use std::ptr;
-use std::str::{self, FromStr};
+use std::str;
 
 use libc;
+use markup5ever::LocalName;
 use url::Url;
 
 use glib::translate::*;
 use glib_sys::{gboolean, gpointer, GList};
 
 use crate::allowed_url::AllowedUrl;
-use crate::attributes::Attribute;
 use crate::croco::*;
 use crate::error::*;
 use crate::io::{self, BinaryData};
-use crate::node::RsvgNode;
+use crate::node::NodeData;
 use crate::properties::{parse_attribute_value_into_parsed_property, ParsedProperty};
 use crate::util::utf8_cstr;
 
 /// A parsed CSS declaration (`name: value [!important]`)
 pub struct Declaration {
-    pub attribute: Attribute,
+    pub attribute: LocalName,
     pub property: ParsedProperty,
     pub important: bool,
 }
 
+#[derive(Default)]
 pub struct DeclarationList {
     // Maps property_name -> Declaration
-    declarations: HashMap<Attribute, Declaration>,
+    declarations: HashMap<LocalName, Declaration>,
 }
 
 pub struct DeclParser;
@@ -44,20 +51,17 @@ impl<'i> DeclarationParser<'i> for DeclParser {
         name: CowRcStr<'i>,
         input: &mut Parser<'i, 't>,
     ) -> Result<Declaration, cssparser::ParseError<'i, ValueErrorKind>> {
-        if let Ok(attribute) = Attribute::from_str(name.as_ref()) {
-            let property = parse_attribute_value_into_parsed_property(attribute, input, true)
-                .map_err(|e| input.new_custom_error(e))?;
+        let attribute = LocalName::from(name.as_ref());
+        let property = parse_attribute_value_into_parsed_property(&attribute, input, true)
+            .map_err(|e| input.new_custom_error(e))?;
 
-            let important = input.try_parse(parse_important).is_ok();
+        let important = input.try_parse(parse_important).is_ok();
 
-            Ok(Declaration {
-                attribute,
-                property,
-                important,
-            })
-        } else {
-            Err(input.new_custom_error(ValueErrorKind::UnknownProperty))
-        }
+        Ok(Declaration {
+            attribute,
+            property,
+            important,
+        })
     }
 }
 
@@ -79,19 +83,14 @@ impl Selector {
 
 /// Contains all the mappings of selectors to style declarations
 /// that result from loading an SVG document.
+#[derive(Default)]
 pub struct CssRules {
     selectors_to_declarations: HashMap<Selector, DeclarationList>,
 }
 
 impl DeclarationList {
-    fn new() -> DeclarationList {
-        DeclarationList {
-            declarations: HashMap::new(),
-        }
-    }
-
     fn add_declaration(&mut self, declaration: Declaration) {
-        match self.declarations.entry(declaration.attribute) {
+        match self.declarations.entry(declaration.attribute.clone()) {
             Entry::Occupied(mut e) => {
                 let decl = e.get_mut();
 
@@ -111,7 +110,7 @@ impl DeclarationList {
     }
 }
 
-pub struct DeclarationListIter<'a>(HashMapIter<'a, Attribute, Declaration>);
+pub struct DeclarationListIter<'a>(HashMapIter<'a, LocalName, Declaration>);
 
 impl<'a> Iterator for DeclarationListIter<'a> {
     type Item = &'a Declaration;
@@ -122,12 +121,6 @@ impl<'a> Iterator for DeclarationListIter<'a> {
 }
 
 impl CssRules {
-    pub fn new() -> CssRules {
-        CssRules {
-            selectors_to_declarations: HashMap::new(),
-        }
-    }
-
     pub fn parse(&mut self, base_url: Option<&Url>, buf: &str) {
         if buf.len() == 0 {
             return; // libcroco doesn't like empty strings :(
@@ -198,7 +191,7 @@ impl CssRules {
         let decl_list = self
             .selectors_to_declarations
             .entry(selector)
-            .or_insert_with(|| DeclarationList::new());
+            .or_insert_with(|| DeclarationList::default());
 
         decl_list.add_declaration(declaration);
     }
@@ -211,7 +204,7 @@ impl CssRules {
         self.selectors_to_declarations.get(selector)
     }
 
-    fn selector_matches_node(&self, selector: &Selector, node: &RsvgNode) -> bool {
+    fn selector_matches_node(&self, selector: &Selector, node_data: &NodeData) -> bool {
         // Try to properly support all of the following, including inheritance:
         // *
         // #id
@@ -222,8 +215,8 @@ impl CssRules {
         //
         // This is basically a semi-compliant CSS2 selection engine
 
-        let element_name = node.get_type().element_name();
-        let id = node.get_id();
+        let element_name = node_data.element_name();
+        let id = node_data.get_id();
 
         // *
         if *selector == Selector::new("*") {
@@ -235,8 +228,8 @@ impl CssRules {
             return true;
         }
 
-        if let Some(klazz) = node.get_class() {
-            for cls in klazz.split_whitespace() {
+        if let Some(class) = node_data.get_class() {
+            for cls in class.split_whitespace() {
                 if !cls.is_empty() {
                     // tag.class#id
                     if let Some(id) = id {
@@ -286,11 +279,11 @@ impl CssRules {
         false
     }
 
-    pub fn get_matches(&self, node: &RsvgNode) -> Vec<Selector> {
+    pub fn get_matches(&self, node_data: &NodeData) -> Vec<Selector> {
         self.selectors_to_declarations
             .iter()
             .filter_map(|(selector, _)| {
-                if self.selector_matches_node(selector, node) {
+                if self.selector_matches_node(selector, node_data) {
                     Some(selector)
                 } else {
                     None
@@ -397,26 +390,25 @@ unsafe extern "C" fn css_property(
 
                 let important = from_glib(a_is_important);
 
-                if let Ok(attribute) = Attribute::from_str(prop_name) {
-                    let mut input = ParserInput::new(&prop_value);
-                    let mut parser = Parser::new(&mut input);
+                let attribute = LocalName::from(prop_name);
 
-                    match parse_attribute_value_into_parsed_property(attribute, &mut parser, true) {
-                        Ok(property) => {
-                            let declaration = Declaration {
-                                attribute,
-                                property,
-                                important,
-                            };
+                let mut input = ParserInput::new(&prop_value);
+                let mut parser = Parser::new(&mut input);
 
-                            handler_data
-                                .css_rules
-                                .add_declaration(Selector::new(&selector_name), declaration);
-                        }
-                        Err(_) => (), // invalid property name or invalid value; ignore
+                match parse_attribute_value_into_parsed_property(&attribute, &mut parser, true) {
+                    Ok(property) => {
+                        let declaration = Declaration {
+                            attribute,
+                            property,
+                            important,
+                        };
+
+                        handler_data
+                            .css_rules
+                            .add_declaration(Selector::new(&selector_name), declaration);
                     }
+                    Err(_) => (), // invalid property name or invalid value; ignore
                 }
-                // else unknown property name; ignore
             }
         }
 
