@@ -1,8 +1,6 @@
-use std::cell::{Cell, RefCell};
-
 use cairo;
+use markup5ever::{local_name, LocalName};
 
-use crate::attributes::Attribute;
 use crate::drawing_ctx::DrawingCtx;
 use crate::error::NodeError;
 use crate::node::{NodeResult, NodeTrait, RsvgNode};
@@ -27,32 +25,34 @@ enum Mode {
 /// The `feBlend` filter primitive.
 pub struct Blend {
     base: PrimitiveWithInput,
-    in2: RefCell<Option<Input>>,
-    mode: Cell<Mode>,
+    in2: Option<Input>,
+    mode: Mode,
 }
 
-impl Blend {
+impl Default for Blend {
     /// Constructs a new `Blend` with empty properties.
     #[inline]
-    pub fn new() -> Blend {
+    fn default() -> Blend {
         Blend {
             base: PrimitiveWithInput::new::<Self>(),
-            in2: RefCell::new(None),
-            mode: Cell::new(Mode::Normal),
+            in2: None,
+            mode: Mode::Normal,
         }
     }
 }
 
 impl NodeTrait for Blend {
-    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
-        self.base.set_atts(node, pbag)?;
+    impl_node_as_filter!();
+
+    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+        self.base.set_atts(parent, pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::In2 => {
-                    self.in2.replace(Some(Input::parse(attr, value)?));
+                local_name!("in2") => {
+                    self.in2 = Some(Input::parse(attr, value)?);
                 }
-                Attribute::Mode => self.mode.set(Mode::parse(attr, value)?),
+                local_name!("mode") => self.mode = Mode::parse(attr, value)?,
                 _ => (),
             }
         }
@@ -69,7 +69,7 @@ impl Filter for Blend {
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
         let input = self.base.get_input(ctx, draw_ctx)?;
-        let input_2 = ctx.get_input(draw_ctx, self.in2.borrow().as_ref())?;
+        let input_2 = ctx.get_input(draw_ctx, self.in2.as_ref())?;
         let bounds = self
             .base
             .get_bounds(ctx)
@@ -106,12 +106,12 @@ impl Filter for Blend {
             cr.clip();
 
             input.surface().set_as_source_surface(&cr, 0f64, 0f64);
-            cr.set_operator(self.mode.get().into());
+            cr.set_operator(self.mode.into());
             cr.paint();
         }
 
         Ok(FilterResult {
-            name: self.base.result.borrow().clone(),
+            name: self.base.result.clone(),
             output: FilterOutput {
                 surface: SharedImageSurface::new(output_surface, surface_type)?,
                 bounds,
@@ -126,7 +126,7 @@ impl Filter for Blend {
 }
 
 impl Mode {
-    fn parse(attr: Attribute, s: &str) -> Result<Self, NodeError> {
+    fn parse(attr: LocalName, s: &str) -> Result<Self, NodeError> {
         match s {
             "normal" => Ok(Mode::Normal),
             "multiply" => Ok(Mode::Multiply),
