@@ -208,11 +208,7 @@ impl Parse for Length {
     type Err = ValueErrorKind;
 
     fn parse(parser: &mut Parser<'_, '_>) -> Result<Length, ValueErrorKind> {
-        let length = Length::from_cssparser(parser)?;
-
-        parser.expect_exhausted().map_err(|_| make_err())?;
-
-        Ok(length)
+        Length::from_cssparser(parser)
     }
 }
 
@@ -360,7 +356,10 @@ impl Parse for Dasharray {
     type Err = ValueErrorKind;
 
     fn parse(parser: &mut Parser<'_, '_>) -> Result<Dasharray, ValueErrorKind> {
-        if parser.try_parse(|p| p.expect_ident_matching("none")).is_ok() {
+        if parser
+            .try_parse(|p| p.expect_ident_matching("none"))
+            .is_ok()
+        {
             Ok(Dasharray::None)
         } else {
             Ok(Dasharray::Array(parse_dash_array(parser)?))
