@@ -5,15 +5,6 @@
 
 use ::xml as xml_rs;
 
-#[macro_use]
-extern crate bitflags;
-
-#[macro_use]
-extern crate glib;
-
-#[macro_use]
-extern crate lazy_static;
-
 pub use crate::c_api::{
     rsvg_rust_error_get_type,
     rsvg_rust_handle_close,
@@ -88,7 +79,6 @@ mod property_macros;
 mod allowed_url;
 mod angle;
 mod aspect_ratio;
-mod attributes;
 mod bbox;
 mod c_api;
 mod clip_path;
@@ -126,14 +116,12 @@ pub mod rect;
 mod shapes;
 mod space;
 pub mod srgb;
-mod stop;
 mod structure;
 mod style;
 pub mod surface_utils;
 mod svg;
 mod text;
 mod transform;
-pub mod tree_utils;
 mod unit_interval;
 mod util;
 mod viewbox;
