@@ -1,14 +1,13 @@
-use std::cell::Cell;
 use std::f64::consts::*;
 use std::ops::Deref;
 
-use cairo::{MatrixTrait, Rectangle};
+use cairo::Rectangle;
 use cssparser::{CowRcStr, Parser, Token};
+use markup5ever::local_name;
 
 use crate::allowed_url::Fragment;
 use crate::angle::Angle;
 use crate::aspect_ratio::*;
-use crate::attributes::Attribute;
 use crate::drawing_ctx::DrawingCtx;
 use crate::error::*;
 use crate::float_eq_cairo::ApproxEqCairo;
@@ -77,7 +76,10 @@ impl Parse for MarkerOrient {
     type Err = ValueErrorKind;
 
     fn parse(parser: &mut Parser<'_, '_>) -> Result<MarkerOrient, ValueErrorKind> {
-        if parser.try_parse(|p| p.expect_ident_matching("auto")).is_ok() {
+        if parser
+            .try_parse(|p| p.expect_ident_matching("auto"))
+            .is_ok()
+        {
             Ok(MarkerOrient::Auto)
         } else {
             Angle::parse(parser).map(MarkerOrient::Angle)
@@ -86,31 +88,33 @@ impl Parse for MarkerOrient {
 }
 
 pub struct NodeMarker {
-    units: Cell<MarkerUnits>,
-    ref_x: Cell<LengthHorizontal>,
-    ref_y: Cell<LengthVertical>,
-    width: Cell<LengthHorizontal>,
-    height: Cell<LengthVertical>,
-    orient: Cell<MarkerOrient>,
-    aspect: Cell<AspectRatio>,
-    vbox: Cell<Option<ViewBox>>,
+    units: MarkerUnits,
+    ref_x: LengthHorizontal,
+    ref_y: LengthVertical,
+    width: LengthHorizontal,
+    height: LengthVertical,
+    orient: MarkerOrient,
+    aspect: AspectRatio,
+    vbox: Option<ViewBox>,
+}
+
+impl Default for NodeMarker {
+    fn default() -> NodeMarker {
+        NodeMarker {
+            units: MarkerUnits::default(),
+            ref_x: Default::default(),
+            ref_y: Default::default(),
+            // the following two are per the spec
+            width: LengthHorizontal::parse_str("3").unwrap(),
+            height: LengthVertical::parse_str("3").unwrap(),
+            orient: MarkerOrient::default(),
+            aspect: AspectRatio::default(),
+            vbox: None,
+        }
+    }
 }
 
 impl NodeMarker {
-    pub fn new() -> NodeMarker {
-        NodeMarker {
-            units: Cell::new(MarkerUnits::default()),
-            ref_x: Cell::new(Default::default()),
-            ref_y: Cell::new(Default::default()),
-            // the following two are per the spec
-            width: Cell::new(LengthHorizontal::parse_str("3").unwrap()),
-            height: Cell::new(LengthVertical::parse_str("3").unwrap()),
-            orient: Cell::new(MarkerOrient::default()),
-            aspect: Cell::new(AspectRatio::default()),
-            vbox: Cell::new(None),
-        }
-    }
-
     fn render(
         &self,
         node: &RsvgNode,
@@ -121,13 +125,13 @@ impl NodeMarker {
         line_width: f64,
         clipping: bool,
     ) -> Result<(), RenderingError> {
-        let cascaded = node.get_cascaded_values();
+        let cascaded = CascadedValues::new_from_node(&node);
         let values = cascaded.get();
 
         let params = draw_ctx.get_view_params();
 
-        let marker_width = self.width.get().normalize(&values, &params);
-        let marker_height = self.height.get().normalize(&values, &params);
+        let marker_width = self.width.normalize(&values, &params);
+        let marker_height = self.height.normalize(&values, &params);
 
         if marker_width.approx_eq_cairo(&0.0) || marker_height.approx_eq_cairo(&0.0) {
             // markerWidth or markerHeight set to 0 disables rendering of the element
@@ -135,52 +139,46 @@ impl NodeMarker {
             return Ok(());
         }
 
-        let cr = draw_ctx.get_cairo_context();
-
-        let mut affine = cr.get_matrix();
-
-        affine.translate(xpos, ypos);
-
-        let rotation = match self.orient.get() {
-            MarkerOrient::Auto => computed_angle,
-            MarkerOrient::Angle(a) => a,
-        };
-
-        affine.rotate(rotation.radians());
-
-        if self.units.get() == MarkerUnits::StrokeWidth {
-            affine.scale(line_width, line_width);
-        }
-
-        let params = if let Some(vbox) = self.vbox.get() {
-            let (_, _, w, h) = self.aspect.get().compute(
-                &vbox,
-                &Rectangle::new(0.0, 0.0, marker_width, marker_height),
-            );
-
-            if vbox.width.approx_eq_cairo(&0.0) || vbox.height.approx_eq_cairo(&0.0) {
-                return Ok(());
-            }
-
-            affine.scale(w / vbox.width, h / vbox.height);
-
-            draw_ctx.push_view_box(vbox.width, vbox.height)
-        } else {
-            draw_ctx.push_view_box(marker_width, marker_height)
-        };
-
-        affine.translate(
-            -self.ref_x.get().normalize(&values, &params),
-            -self.ref_y.get().normalize(&values, &params),
-        );
-
         draw_ctx.with_saved_cr(&mut |dc| {
             let cr = dc.get_cairo_context();
 
-            cr.set_matrix(affine);
+            cr.translate(xpos, ypos);
+
+            let rotation = match self.orient {
+                MarkerOrient::Auto => computed_angle,
+                MarkerOrient::Angle(a) => a,
+            };
+
+            cr.rotate(rotation.radians());
+
+            if self.units == MarkerUnits::StrokeWidth {
+                cr.scale(line_width, line_width);
+            }
+
+            let params = if let Some(vbox) = self.vbox {
+                let (_, _, w, h) = self.aspect.compute(
+                    &vbox,
+                    &Rectangle::new(0.0, 0.0, marker_width, marker_height),
+                );
+
+                if vbox.width.approx_eq_cairo(&0.0) || vbox.height.approx_eq_cairo(&0.0) {
+                    return Ok(());
+                }
+
+                cr.scale(w / vbox.width, h / vbox.height);
+
+                dc.push_view_box(vbox.width, vbox.height)
+            } else {
+                dc.push_view_box(marker_width, marker_height)
+            };
+
+            cr.translate(
+                -self.ref_x.normalize(&values, &params),
+                -self.ref_y.normalize(&values, &params),
+            );
 
             if !values.is_overflow() {
-                if let Some(vbox) = self.vbox.get() {
+                if let Some(vbox) = self.vbox {
                     dc.clip(vbox.x, vbox.y, vbox.width, vbox.height);
                 } else {
                     dc.clip(0.0, 0.0, marker_width, marker_height);
@@ -195,33 +193,23 @@ impl NodeMarker {
 }
 
 impl NodeTrait for NodeMarker {
-    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
-        // marker element has overflow:hidden
-        // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
-        node.set_overflow_hidden();
-
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::MarkerUnits => self.units.set(attr.parse(value)?),
-
-                Attribute::RefX => self.ref_x.set(attr.parse(value)?),
-
-                Attribute::RefY => self.ref_y.set(attr.parse(value)?),
-
-                Attribute::MarkerWidth => self
-                    .width
-                    .set(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?),
-
-                Attribute::MarkerHeight => self
-                    .height
-                    .set(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?),
-
-                Attribute::Orient => self.orient.set(attr.parse(value)?),
-
-                Attribute::PreserveAspectRatio => self.aspect.set(attr.parse(value)?),
-
-                Attribute::ViewBox => self.vbox.set(Some(attr.parse(value)?)),
-
+                local_name!("markerUnits") => self.units = attr.parse(value)?,
+                local_name!("refX") => self.ref_x = attr.parse(value)?,
+                local_name!("refY") => self.ref_y = attr.parse(value)?,
+                local_name!("markerWidth") => {
+                    self.width =
+                        attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?
+                }
+                local_name!("markerHeight") => {
+                    self.height =
+                        attr.parse_and_validate(value, LengthVertical::check_nonnegative)?
+                }
+                local_name!("orient") => self.orient = attr.parse(value)?,
+                local_name!("preserveAspectRatio") => self.aspect = attr.parse(value)?,
+                local_name!("viewBox") => self.vbox = Some(attr.parse(value)?),
                 _ => (),
             }
         }
@@ -229,6 +217,10 @@ impl NodeTrait for NodeMarker {
         Ok(())
     }
 
+    fn overflow_hidden(&self) -> bool {
+        true
+    }
+
     fn set_overridden_properties(&self, values: &mut SpecifiedValues) {
         // markers are always displayed, even if <marker> or its ancestors are display:none
         values.display = SpecifiedValue::Specified(Default::default());
@@ -601,17 +593,15 @@ fn emit_marker_by_name(
     {
         let node = acquired.get();
 
-        node.with_impl(|marker: &NodeMarker| {
-            marker.render(
-                &node,
-                draw_ctx,
-                xpos,
-                ypos,
-                computed_angle,
-                line_width,
-                clipping,
-            )
-        })
+        node.borrow().get_impl::<NodeMarker>().render(
+            &node,
+            draw_ctx,
+            xpos,
+            ypos,
+            computed_angle,
+            line_width,
+            clipping,
+        )
     } else {
         rsvg_log!("marker \"{}\" not found", name);
         Ok(())
