@@ -4,7 +4,7 @@ use crate::allowed_url::Fragment;
 use crate::bbox::BoundingBox;
 use crate::drawing_ctx::DrawingCtx;
 use crate::error::*;
-use crate::node::RsvgNode;
+use crate::node::{CascadedValues, RsvgNode};
 use crate::parsers::Parse;
 use crate::properties::ComputedValues;
 use crate::unit_interval::UnitInterval;
@@ -23,11 +23,17 @@ impl Parse for PaintServer {
     type Err = ValueErrorKind;
 
     fn parse(parser: &mut Parser<'_, '_>) -> Result<PaintServer, ValueErrorKind> {
-        if parser.try_parse(|i| i.expect_ident_matching("none")).is_ok() {
+        if parser
+            .try_parse(|i| i.expect_ident_matching("none"))
+            .is_ok()
+        {
             Ok(PaintServer::None)
         } else if let Ok(url) = parser.try_parse(|i| i.expect_url()) {
             let alternate = if !parser.is_exhausted() {
-                if parser.try_parse(|i| i.expect_ident_matching("none")).is_ok() {
+                if parser
+                    .try_parse(|i| i.expect_ident_matching("none"))
+                    .is_ok()
+                {
                     None
                 } else {
                     Some(parser.try_parse(|i| cssparser::Color::parse(i))?)
@@ -75,7 +81,7 @@ pub trait PaintSource {
         bbox: &BoundingBox,
     ) -> Result<bool, RenderingError> {
         if let Some(resolved) = self.resolve(&node, draw_ctx, bbox)? {
-            let cascaded = node.get_cascaded_values();
+            let cascaded = CascadedValues::new_from_node(node);
             let values = cascaded.get();
             self.set_pattern_on_draw_context(&resolved, values, draw_ctx, opacity, bbox)
         } else {
@@ -84,6 +90,17 @@ pub trait PaintSource {
     }
 }
 
+// Any of the attributes in gradient and pattern elements may be omitted.
+// The missing ones are resolved from the "fallback" IRI. If still missing,
+// they are resolved to the default value
+pub trait Resolve {
+    fn is_resolved(&self) -> bool;
+
+    fn resolve_from_fallback(&mut self, fallback: &Self);
+
+    fn resolve_from_defaults(&mut self);
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
