@@ -1,9 +1,7 @@
 use cairo;
-use std::cell::Cell;
-use std::cell::RefCell;
+use markup5ever::local_name;
 use std::ops::Deref;
 
-use crate::attributes::Attribute;
 use crate::drawing_ctx::DrawingCtx;
 use crate::error::*;
 use crate::length::*;
@@ -110,32 +108,25 @@ fn render_ellipse(
     render_path_builder(&builder, draw_ctx, node, values, false, clipping)
 }
 
+#[derive(Default)]
 pub struct NodePath {
-    builder: RefCell<Option<PathBuilder>>,
-}
-
-impl NodePath {
-    pub fn new() -> NodePath {
-        NodePath {
-            builder: RefCell::new(None),
-        }
-    }
+    builder: Option<PathBuilder>,
 }
 
 impl NodeTrait for NodePath {
-    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
-            if attr == Attribute::D {
+            if attr == local_name!("d") {
                 let mut builder = PathBuilder::new();
 
                 if let Err(e) = path_parser::parse_path_into_builder(value, &mut builder) {
                     // FIXME: we don't propagate errors upstream, but creating a partial
                     // path is OK per the spec
 
-                    rsvg_log!("could not parse path {}: {}", node, e);
+                    rsvg_log!("could not parse path: {}", e);
                 }
 
-                *self.builder.borrow_mut() = Some(builder);
+                self.builder = Some(builder);
             }
         }
 
@@ -151,7 +142,7 @@ impl NodeTrait for NodePath {
     ) -> Result<(), RenderingError> {
         let values = cascaded.get();
 
-        if let Some(ref builder) = *self.builder.borrow() {
+        if let Some(ref builder) = self.builder {
             render_path_builder(builder, draw_ctx, node, values, true, clipping)?;
         }
 
@@ -206,32 +197,31 @@ impl Parse for Points {
 }
 
 pub struct NodePoly {
-    points: RefCell<Option<Points>>,
+    points: Option<Points>,
     kind: PolyKind,
 }
 
 impl NodePoly {
     pub fn new_open() -> NodePoly {
         NodePoly {
-            points: RefCell::new(None),
+            points: None,
             kind: PolyKind::Open,
         }
     }
 
     pub fn new_closed() -> NodePoly {
         NodePoly {
-            points: RefCell::new(None),
+            points: None,
             kind: PolyKind::Closed,
         }
     }
 }
 
 impl NodeTrait for NodePoly {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
-            // support for svg < 1.0 which used verts
-            if attr == Attribute::Points || attr == Attribute::Verts {
-                *self.points.borrow_mut() = attr.parse(value.trim()).map(Some)?;
+            if attr == local_name!("points") {
+                self.points = attr.parse(value.trim()).map(Some)?;
             }
         }
 
@@ -247,7 +237,7 @@ impl NodeTrait for NodePoly {
     ) -> Result<(), RenderingError> {
         let values = cascaded.get();
 
-        if let Some(ref points) = *self.points.borrow() {
+        if let Some(ref points) = self.points {
             let mut builder = PathBuilder::new();
 
             for (i, &(x, y)) in points.iter().enumerate() {
@@ -269,32 +259,22 @@ impl NodeTrait for NodePoly {
     }
 }
 
+#[derive(Default)]
 pub struct NodeLine {
-    x1: Cell<LengthHorizontal>,
-    y1: Cell<LengthVertical>,
-    x2: Cell<LengthHorizontal>,
-    y2: Cell<LengthVertical>,
-}
-
-impl NodeLine {
-    pub fn new() -> NodeLine {
-        NodeLine {
-            x1: Cell::new(Default::default()),
-            y1: Cell::new(Default::default()),
-            x2: Cell::new(Default::default()),
-            y2: Cell::new(Default::default()),
-        }
-    }
+    x1: LengthHorizontal,
+    y1: LengthVertical,
+    x2: LengthHorizontal,
+    y2: LengthVertical,
 }
 
 impl NodeTrait for NodeLine {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X1 => self.x1.set(attr.parse(value)?),
-                Attribute::Y1 => self.y1.set(attr.parse(value)?),
-                Attribute::X2 => self.x2.set(attr.parse(value)?),
-                Attribute::Y2 => self.y2.set(attr.parse(value)?),
+                local_name!("x1") => self.x1 = attr.parse(value)?,
+                local_name!("y1") => self.y1 = attr.parse(value)?,
+                local_name!("x2") => self.x2 = attr.parse(value)?,
+                local_name!("y2") => self.y2 = attr.parse(value)?,
                 _ => (),
             }
         }
@@ -315,10 +295,10 @@ impl NodeTrait for NodeLine {
 
         let params = draw_ctx.get_view_params();
 
-        let x1 = self.x1.get().normalize(values, &params);
-        let y1 = self.y1.get().normalize(values, &params);
-        let x2 = self.x2.get().normalize(values, &params);
-        let y2 = self.y2.get().normalize(values, &params);
+        let x1 = self.x1.normalize(values, &params);
+        let y1 = self.y1.normalize(values, &params);
+        let x2 = self.x2.normalize(values, &params);
+        let y2 = self.y2.normalize(values, &params);
 
         builder.move_to(x1, y1);
         builder.line_to(x2, y2);
@@ -327,53 +307,40 @@ impl NodeTrait for NodeLine {
     }
 }
 
+#[derive(Default)]
 pub struct NodeRect {
-    // x, y, width, height
-    x: Cell<LengthHorizontal>,
-    y: Cell<LengthVertical>,
-    w: Cell<LengthHorizontal>,
-    h: Cell<LengthVertical>,
+    x: LengthHorizontal,
+    y: LengthVertical,
+    w: LengthHorizontal,
+    h: LengthVertical,
 
     // Radiuses for rounded corners
-    rx: Cell<Option<LengthHorizontal>>,
-    ry: Cell<Option<LengthVertical>>,
-}
-
-impl NodeRect {
-    pub fn new() -> NodeRect {
-        NodeRect {
-            x: Cell::new(Default::default()),
-            y: Cell::new(Default::default()),
-            w: Cell::new(Default::default()),
-            h: Cell::new(Default::default()),
-
-            rx: Cell::new(None),
-            ry: Cell::new(None),
-        }
-    }
+    rx: Option<LengthHorizontal>,
+    ry: Option<LengthVertical>,
 }
 
 impl NodeTrait for NodeRect {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(attr.parse(value)?),
-                Attribute::Y => self.y.set(attr.parse(value)?),
-                Attribute::Width => self
-                    .w
-                    .set(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?),
-                Attribute::Height => self
-                    .h
-                    .set(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?),
-                Attribute::Rx => self.rx.set(
-                    attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)
-                        .map(Some)?,
-                ),
-                Attribute::Ry => self.ry.set(
-                    attr.parse_and_validate(value, LengthVertical::check_nonnegative)
-                        .map(Some)?,
-                ),
-
+                local_name!("x") => self.x = attr.parse(value)?,
+                local_name!("y") => self.y = attr.parse(value)?,
+                local_name!("width") => {
+                    self.w = attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?
+                }
+                local_name!("height") => {
+                    self.h = attr.parse_and_validate(value, LengthVertical::check_nonnegative)?
+                }
+                local_name!("rx") => {
+                    self.rx = attr
+                        .parse_and_validate(value, LengthHorizontal::check_nonnegative)
+                        .map(Some)?
+                }
+                local_name!("ry") => {
+                    self.ry = attr
+                        .parse_and_validate(value, LengthVertical::check_nonnegative)
+                        .map(Some)?
+                }
                 _ => (),
             }
         }
@@ -392,15 +359,15 @@ impl NodeTrait for NodeRect {
 
         let params = draw_ctx.get_view_params();
 
-        let x = self.x.get().normalize(values, &params);
-        let y = self.y.get().normalize(values, &params);
-        let w = self.w.get().normalize(values, &params);
-        let h = self.h.get().normalize(values, &params);
+        let x = self.x.normalize(values, &params);
+        let y = self.y.normalize(values, &params);
+        let w = self.w.normalize(values, &params);
+        let h = self.h.normalize(values, &params);
 
         let mut rx;
         let mut ry;
 
-        match (self.rx.get(), self.ry.get()) {
+        match (self.rx, self.ry) {
             (None, None) => {
                 rx = 0.0;
                 ry = 0.0;
@@ -561,32 +528,22 @@ impl NodeTrait for NodeRect {
     }
 }
 
+#[derive(Default)]
 pub struct NodeCircle {
-    cx: Cell<LengthHorizontal>,
-    cy: Cell<LengthVertical>,
-    r: Cell<LengthBoth>,
-}
-
-impl NodeCircle {
-    pub fn new() -> NodeCircle {
-        NodeCircle {
-            cx: Cell::new(Default::default()),
-            cy: Cell::new(Default::default()),
-            r: Cell::new(Default::default()),
-        }
-    }
+    cx: LengthHorizontal,
+    cy: LengthVertical,
+    r: LengthBoth,
 }
 
 impl NodeTrait for NodeCircle {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::Cx => self.cx.set(attr.parse(value)?),
-                Attribute::Cy => self.cy.set(attr.parse(value)?),
-                Attribute::R => self
-                    .r
-                    .set(attr.parse_and_validate(value, LengthBoth::check_nonnegative)?),
-
+                local_name!("cx") => self.cx = attr.parse(value)?,
+                local_name!("cy") => self.cy = attr.parse(value)?,
+                local_name!("r") => {
+                    self.r = attr.parse_and_validate(value, LengthBoth::check_nonnegative)?
+                }
                 _ => (),
             }
         }
@@ -605,45 +562,34 @@ impl NodeTrait for NodeCircle {
 
         let params = draw_ctx.get_view_params();
 
-        let cx = self.cx.get().normalize(values, &params);
-        let cy = self.cy.get().normalize(values, &params);
-        let r = self.r.get().normalize(values, &params);
+        let cx = self.cx.normalize(values, &params);
+        let cy = self.cy.normalize(values, &params);
+        let r = self.r.normalize(values, &params);
 
         render_ellipse(cx, cy, r, r, draw_ctx, node, values, clipping)
     }
 }
 
+#[derive(Default)]
 pub struct NodeEllipse {
-    cx: Cell<LengthHorizontal>,
-    cy: Cell<LengthVertical>,
-    rx: Cell<LengthHorizontal>,
-    ry: Cell<LengthVertical>,
-}
-
-impl NodeEllipse {
-    pub fn new() -> NodeEllipse {
-        NodeEllipse {
-            cx: Cell::new(Default::default()),
-            cy: Cell::new(Default::default()),
-            rx: Cell::new(Default::default()),
-            ry: Cell::new(Default::default()),
-        }
-    }
+    cx: LengthHorizontal,
+    cy: LengthVertical,
+    rx: LengthHorizontal,
+    ry: LengthVertical,
 }
 
 impl NodeTrait for NodeEllipse {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::Cx => self.cx.set(attr.parse(value)?),
-                Attribute::Cy => self.cy.set(attr.parse(value)?),
-                Attribute::Rx => self
-                    .rx
-                    .set(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?),
-                Attribute::Ry => self
-                    .ry
-                    .set(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?),
-
+                local_name!("cx") => self.cx = attr.parse(value)?,
+                local_name!("cy") => self.cy = attr.parse(value)?,
+                local_name!("rx") => {
+                    self.rx = attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?
+                }
+                local_name!("ry") => {
+                    self.ry = attr.parse_and_validate(value, LengthVertical::check_nonnegative)?
+                }
                 _ => (),
             }
         }
@@ -662,10 +608,10 @@ impl NodeTrait for NodeEllipse {
 
         let params = draw_ctx.get_view_params();
 
-        let cx = self.cx.get().normalize(values, &params);
-        let cy = self.cy.get().normalize(values, &params);
-        let rx = self.rx.get().normalize(values, &params);
-        let ry = self.ry.get().normalize(values, &params);
+        let cx = self.cx.normalize(values, &params);
+        let cy = self.cy.normalize(values, &params);
+        let rx = self.rx.normalize(values, &params);
+        let ry = self.ry.normalize(values, &params);
 
         render_ellipse(cx, cy, rx, ry, draw_ctx, node, values, clipping)
     }
