@@ -1,11 +1,8 @@
-use std::cell::Cell;
-use std::cell::RefCell;
-
 use cairo::Rectangle;
+use markup5ever::local_name;
 
 use crate::allowed_url::Fragment;
 use crate::aspect_ratio::*;
-use crate::attributes::Attribute;
 use crate::dpi::Dpi;
 use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
 use crate::error::{AttributeResultExt, RenderingError};
@@ -17,19 +14,13 @@ use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
 use crate::property_defs::Overflow;
 use crate::rect::RectangleExt;
-use crate::tree_utils::Node;
 use crate::viewbox::*;
 
+#[derive(Default)]
 pub struct NodeGroup();
 
-impl NodeGroup {
-    pub fn new() -> NodeGroup {
-        NodeGroup()
-    }
-}
-
 impl NodeTrait for NodeGroup {
-    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 
@@ -48,30 +39,24 @@ impl NodeTrait for NodeGroup {
     }
 }
 
-pub struct NodeDefs();
+/// A no-op node that does not render anything
+///
+/// Sometimes we just need a node that can contain children, but doesn't
+/// render itself or its children.  This is just that kind of node.
+#[derive(Default)]
+pub struct NodeNonRendering;
 
-impl NodeDefs {
-    pub fn new() -> NodeDefs {
-        NodeDefs()
-    }
-}
-
-impl NodeTrait for NodeDefs {
-    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
+impl NodeTrait for NodeNonRendering {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 }
 
+#[derive(Default)]
 pub struct NodeSwitch();
 
-impl NodeSwitch {
-    pub fn new() -> NodeSwitch {
-        NodeSwitch()
-    }
-}
-
 impl NodeTrait for NodeSwitch {
-    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 
@@ -87,8 +72,8 @@ impl NodeTrait for NodeSwitch {
         draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
             if let Some(child) = node
                 .children()
-                .filter(|c| c.get_type() != NodeType::Chars)
-                .find(|c| c.get_cond())
+                .filter(|c| c.borrow().get_type() != NodeType::Chars)
+                .find(|c| c.borrow().get_cond())
             {
                 dc.draw_node_from_stack(&CascadedValues::new(cascaded, &child), &child, clipping)
             } else {
@@ -106,31 +91,21 @@ pub struct IntrinsicDimensions {
     pub vbox: Option<ViewBox>,
 }
 
+#[derive(Default)]
 pub struct NodeSvg {
-    preserve_aspect_ratio: Cell<AspectRatio>,
-    x: Cell<Option<LengthHorizontal>>,
-    y: Cell<Option<LengthVertical>>,
-    w: Cell<Option<LengthHorizontal>>,
-    h: Cell<Option<LengthVertical>>,
-    vbox: Cell<Option<ViewBox>>,
+    preserve_aspect_ratio: AspectRatio,
+    x: Option<LengthHorizontal>,
+    y: Option<LengthVertical>,
+    w: Option<LengthHorizontal>,
+    h: Option<LengthVertical>,
+    vbox: Option<ViewBox>,
 }
 
 impl NodeSvg {
-    pub fn new() -> NodeSvg {
-        NodeSvg {
-            preserve_aspect_ratio: Cell::new(AspectRatio::default()),
-            x: Cell::new(None),
-            y: Cell::new(None),
-            w: Cell::new(None),
-            h: Cell::new(None),
-            vbox: Cell::new(None),
-        }
-    }
-
     pub fn get_size(&self, values: &ComputedValues, dpi: Dpi) -> Option<(i32, i32)> {
         let (_, _, w, h) = self.get_unnormalized_viewport();
 
-        match (w, h, self.vbox.get()) {
+        match (w, h, self.vbox) {
             (w, h, Some(vbox)) => {
                 let params = ViewParams::new(dpi.x(), dpi.y(), vbox.width, vbox.height);
 
@@ -154,9 +129,9 @@ impl NodeSvg {
 
     pub fn get_intrinsic_dimensions(&self) -> IntrinsicDimensions {
         IntrinsicDimensions {
-            width: self.w.get(),
-            height: self.h.get(),
-            vbox: self.vbox.get(),
+            width: self.w,
+            height: self.h,
+            vbox: self.vbox,
         }
     }
 
@@ -172,19 +147,15 @@ impl NodeSvg {
         // these defaults are per the spec
         let x = self
             .x
-            .get()
             .unwrap_or_else(|| LengthHorizontal::parse_str("0").unwrap());
         let y = self
             .y
-            .get()
             .unwrap_or_else(|| LengthVertical::parse_str("0").unwrap());
         let w = self
             .w
-            .get()
             .unwrap_or_else(|| LengthHorizontal::parse_str("100%").unwrap());
         let h = self
             .h
-            .get()
             .unwrap_or_else(|| LengthVertical::parse_str("100%").unwrap());
 
         (x, y, w, h)
@@ -203,43 +174,27 @@ impl NodeSvg {
 }
 
 impl NodeTrait for NodeSvg {
-    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
-        // SVG element has overflow:hidden
-        // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
-        node.set_overflow_hidden();
-
+    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         // x & y attributes have no effect on outermost svg
         // http://www.w3.org/TR/SVG/struct.html#SVGElement
-        let is_inner_svg = node.parent().is_some();
+        let is_inner_svg = parent.is_some();
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::PreserveAspectRatio => {
-                    self.preserve_aspect_ratio.set(attr.parse(value)?)
+                local_name!("preserveAspectRatio") => {
+                    self.preserve_aspect_ratio = attr.parse(value)?
                 }
-
-                Attribute::X => {
-                    if is_inner_svg {
-                        self.x.set(Some(attr.parse(value)?));
-                    }
+                local_name!("x") if is_inner_svg => self.x = Some(attr.parse(value)?),
+                local_name!("y") if is_inner_svg => self.y = Some(attr.parse(value)?),
+                local_name!("width") => {
+                    self.w =
+                        Some(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?)
                 }
-
-                Attribute::Y => {
-                    if is_inner_svg {
-                        self.y.set(Some(attr.parse(value)?));
-                    }
+                local_name!("height") => {
+                    self.h =
+                        Some(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?)
                 }
-
-                Attribute::Width => self.w.set(Some(
-                    attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?,
-                )),
-
-                Attribute::Height => self.h.set(Some(
-                    attr.parse_and_validate(value, LengthVertical::check_nonnegative)?,
-                )),
-
-                Attribute::ViewBox => self.vbox.set(attr.parse(value).map(Some)?),
-
+                local_name!("viewBox") => self.vbox = attr.parse(value).map(Some)?,
                 _ => (),
             }
         }
@@ -247,6 +202,10 @@ impl NodeTrait for NodeSvg {
         Ok(())
     }
 
+    fn overflow_hidden(&self) -> bool {
+        true
+    }
+
     fn draw(
         &self,
         node: &RsvgNode,
@@ -274,18 +233,17 @@ impl NodeTrait for NodeSvg {
             // We are obtaining the toplevel SVG's geometry.  This means, don't care about the
             // DrawingCtx's viewport, just use the SVG's intrinsic dimensions and see how far
             // it wants to extend.
-            (svg_viewport, self.vbox.get())
+            (svg_viewport, self.vbox)
         } else {
             if has_parent {
-                (svg_viewport, self.vbox.get())
+                (svg_viewport, self.vbox)
             } else {
                 (
                     // The client's viewport overrides the toplevel's x/y/w/h viewport
                     draw_ctx.toplevel_viewport(),
-
                     // Use our viewBox if available, or try to derive one from
                     // the intrinsic dimensions.
-                    self.vbox.get().or_else(|| {
+                    self.vbox.or_else(|| {
                         Some(ViewBox {
                             x: 0.0,
                             y: 0.0,
@@ -299,54 +257,41 @@ impl NodeTrait for NodeSvg {
 
         draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
             let _params =
-                dc.push_new_viewport(vbox, &viewport, self.preserve_aspect_ratio.get(), clip_mode);
+                dc.push_new_viewport(vbox, &viewport, self.preserve_aspect_ratio, clip_mode);
 
             node.draw_children(cascaded, dc, clipping)
         })
     }
 }
 
+#[derive(Default)]
 pub struct NodeUse {
-    link: RefCell<Option<Fragment>>,
-    x: Cell<LengthHorizontal>,
-    y: Cell<LengthVertical>,
-    w: Cell<Option<LengthHorizontal>>,
-    h: Cell<Option<LengthVertical>>,
-}
-
-impl NodeUse {
-    pub fn new() -> NodeUse {
-        NodeUse {
-            link: RefCell::new(None),
-            x: Cell::new(Default::default()),
-            y: Cell::new(Default::default()),
-            w: Cell::new(None),
-            h: Cell::new(None),
-        }
-    }
+    link: Option<Fragment>,
+    x: LengthHorizontal,
+    y: LengthVertical,
+    w: Option<LengthHorizontal>,
+    h: Option<LengthVertical>,
 }
 
 impl NodeTrait for NodeUse {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::XlinkHref => {
-                    *self.link.borrow_mut() =
-                        Some(Fragment::parse(value).attribute(Attribute::XlinkHref)?)
+                local_name!("xlink:href") => {
+                    self.link = Some(Fragment::parse(value).attribute(attr)?)
+                }
+                local_name!("x") => self.x = attr.parse(value)?,
+                local_name!("y") => self.y = attr.parse(value)?,
+                local_name!("width") => {
+                    self.w = attr
+                        .parse_and_validate(value, LengthHorizontal::check_nonnegative)
+                        .map(Some)?
+                }
+                local_name!("height") => {
+                    self.h = attr
+                        .parse_and_validate(value, LengthVertical::check_nonnegative)
+                        .map(Some)?
                 }
-
-                Attribute::X => self.x.set(attr.parse(value)?),
-                Attribute::Y => self.y.set(attr.parse(value)?),
-
-                Attribute::Width => self.w.set(
-                    attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)
-                        .map(Some)?,
-                ),
-                Attribute::Height => self.h.set(
-                    attr.parse_and_validate(value, LengthVertical::check_nonnegative)
-                        .map(Some)?,
-                ),
-
                 _ => (),
             }
         }
@@ -363,13 +308,11 @@ impl NodeTrait for NodeUse {
     ) -> Result<(), RenderingError> {
         let values = cascaded.get();
 
-        let link = self.link.borrow();
-
-        if link.is_none() {
+        if self.link.is_none() {
             return Ok(());
         }
 
-        let link = link.as_ref().unwrap();
+        let link = self.link.as_ref().unwrap();
 
         let child = if let Some(acquired) = draw_ctx.acquired_nodes().get_node(link) {
             // Here we clone the acquired child, so that we can drop the AcquiredNode as
@@ -381,7 +324,7 @@ impl NodeTrait for NodeUse {
             return Ok(());
         };
 
-        if Node::is_ancestor(child.clone(), node.clone()) {
+        if node.ancestors().any(|ancestor| ancestor == child) {
             // or, if we're <use>'ing ourselves
             return Err(RenderingError::CircularReference);
         }
@@ -390,8 +333,8 @@ impl NodeTrait for NodeUse {
 
         let params = draw_ctx.get_view_params();
 
-        let nx = self.x.get().normalize(values, &params);
-        let ny = self.y.get().normalize(values, &params);
+        let nx = self.x.normalize(values, &params);
+        let ny = self.y.normalize(values, &params);
 
         // If attributes ‘width’ and/or ‘height’ are not specified,
         // [...] use values of '100%' for these attributes.
@@ -400,12 +343,10 @@ impl NodeTrait for NodeUse {
 
         let nw = self
             .w
-            .get()
             .unwrap_or_else(|| LengthHorizontal::parse_str("100%").unwrap())
             .normalize(values, &params);
         let nh = self
             .h
-            .get()
             .unwrap_or_else(|| LengthVertical::parse_str("100%").unwrap())
             .normalize(values, &params);
 
@@ -417,7 +358,7 @@ impl NodeTrait for NodeUse {
 
         let viewport = Rectangle::new(nx, ny, nw, nh);
 
-        if child.get_type() != NodeType::Symbol {
+        if child.borrow().get_type() != NodeType::Symbol {
             let cr = draw_ctx.get_cairo_context();
             cr.translate(viewport.x, viewport.y);
 
@@ -429,66 +370,57 @@ impl NodeTrait for NodeUse {
                 )
             })
         } else {
-            child.with_impl(|symbol: &NodeSymbol| {
-                let clip_mode = if !values.is_overflow()
-                    || (values.overflow == Overflow::Visible && child.is_overflow())
-                {
-                    Some(ClipMode::ClipToVbox)
-                } else {
-                    None
-                };
+            let node_data = child.borrow();
+            let symbol = node_data.get_impl::<NodeSymbol>();
 
-                draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
-                    let _params = dc.push_new_viewport(
-                        symbol.vbox.get(),
-                        &viewport,
-                        symbol.preserve_aspect_ratio.get(),
-                        clip_mode,
-                    );
+            let clip_mode = if !values.is_overflow()
+                || (values.overflow == Overflow::Visible && child.borrow().is_overflow())
+            {
+                Some(ClipMode::ClipToVbox)
+            } else {
+                None
+            };
 
-                    child.draw_children(
-                        &CascadedValues::new_from_values(&child, values),
-                        dc,
-                        clipping,
-                    )
-                })
+            draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+                let _params = dc.push_new_viewport(
+                    symbol.vbox,
+                    &viewport,
+                    symbol.preserve_aspect_ratio,
+                    clip_mode,
+                );
+
+                child.draw_children(
+                    &CascadedValues::new_from_values(&child, values),
+                    dc,
+                    clipping,
+                )
             })
         }
     }
 }
 
+#[derive(Default)]
 pub struct NodeSymbol {
-    preserve_aspect_ratio: Cell<AspectRatio>,
-    vbox: Cell<Option<ViewBox>>,
-}
-
-impl NodeSymbol {
-    pub fn new() -> NodeSymbol {
-        NodeSymbol {
-            preserve_aspect_ratio: Cell::new(AspectRatio::default()),
-            vbox: Cell::new(None),
-        }
-    }
+    preserve_aspect_ratio: AspectRatio,
+    vbox: Option<ViewBox>,
 }
 
 impl NodeTrait for NodeSymbol {
-    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
-        // symbol element has overflow:hidden
-        // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
-        node.set_overflow_hidden();
-
+    fn set_atts(&mut self, _parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::PreserveAspectRatio => {
-                    self.preserve_aspect_ratio.set(attr.parse(value)?)
+                local_name!("preserveAspectRatio") => {
+                    self.preserve_aspect_ratio = attr.parse(value)?
                 }
-
-                Attribute::ViewBox => self.vbox.set(attr.parse(value).map(Some)?),
-
+                local_name!("viewBox") => self.vbox = attr.parse(value).map(Some)?,
                 _ => (),
             }
         }
 
         Ok(())
     }
+
+    fn overflow_hidden(&self) -> bool {
+        true
+    }
 }
