@@ -604,18 +604,8 @@ dimensions_and_position (void)
     g_assert_cmpint (pos.x, ==, EXAMPLE_TWO_X);
     g_assert_cmpint (pos.y, ==, EXAMPLE_TWO_Y);
 
-    /* TODO: test logical_rect */
-    RsvgRectangle ink_rect;
-    RsvgRectangle logical_rect;
-    g_assert (rsvg_handle_get_geometry_sub (handle, &ink_rect, &logical_rect, EXAMPLE_TWO_ID));
-    g_assert_cmpint (ink_rect.x, ==, EXAMPLE_TWO_X);
-    g_assert_cmpint (ink_rect.y, ==, EXAMPLE_TWO_Y);
-    g_assert_cmpint (ink_rect.width,  ==, EXAMPLE_TWO_W);
-    g_assert_cmpint (ink_rect.height, ==, EXAMPLE_TWO_H);
-
     g_assert (!rsvg_handle_get_position_sub (handle, &pos, EXAMPLE_NONEXISTENT_ID));
     g_assert (!rsvg_handle_get_dimensions_sub (handle, &dim, EXAMPLE_NONEXISTENT_ID));
-    g_assert (!rsvg_handle_get_geometry_sub (handle, &ink_rect, &logical_rect, EXAMPLE_NONEXISTENT_ID));
 
     g_object_unref (handle);
 }
@@ -623,6 +613,7 @@ dimensions_and_position (void)
 struct size_func_data {
     gboolean called;
     gboolean destroyed;
+    gboolean testing_size_func_calls;
 };
 
 static void
@@ -630,10 +621,12 @@ size_func (gint *width, gint *height, gpointer user_data)
 {
     struct size_func_data *data = user_data;
 
-    g_assert (!data->called);
-    data->called = TRUE;
+    if (data->testing_size_func_calls) {
+        g_assert (!data->called);
+        data->called = TRUE;
 
-    g_assert (!data->destroyed);
+        g_assert (!data->destroyed);
+    }
 
     *width = 42;
     *height = 43;
@@ -644,8 +637,10 @@ size_func_destroy (gpointer user_data)
 {
     struct size_func_data *data = user_data;
 
-    g_assert (!data->destroyed);
-    data->destroyed = TRUE;
+    if (data->testing_size_func_calls) {
+        g_assert (!data->destroyed);
+        data->destroyed = TRUE;
+    }
 }
 
 static void
@@ -665,6 +660,7 @@ set_size_callback (void)
 
     data.called = FALSE;
     data.destroyed = FALSE;
+    data.testing_size_func_calls = TRUE;
 
     rsvg_handle_set_size_callback (handle, size_func, &data, size_func_destroy);
 
@@ -695,11 +691,13 @@ reset_size_callback (void)
 
     data_1.called = FALSE;
     data_1.destroyed = FALSE;
+    data_1.testing_size_func_calls = TRUE;
 
     rsvg_handle_set_size_callback (handle, size_func, &data_1, size_func_destroy);
 
     data_2.called = FALSE;
     data_2.destroyed = FALSE;
+    data_2.testing_size_func_calls = TRUE;
 
     rsvg_handle_set_size_callback (handle, size_func, &data_2, size_func_destroy);
     g_assert (data_1.destroyed);
@@ -709,6 +707,88 @@ reset_size_callback (void)
     g_assert (data_2.destroyed);
 }
 
+static void
+zero_size_func (gint *width, gint *height, gpointer user_data)
+{
+    *width = 0;
+    *height = 0;
+}
+
+static void
+render_with_zero_size_callback (void)
+{
+    /* gdk_pixbuf_get_file_info() uses a GdkPixbufLoader, but in its
+     * "size-prepared" callback it saves the computed size, and then calls
+     * gdk_pixbuf_loader_set_size(loader, 0, 0).  Presumably it does to tell
+     * loaders that it only wanted to know the size, but that they shouldn't
+     * decode or render the image to a pixbuf buffer.
+     *
+     * Librsvg used to panic when getting (0, 0) from the size_callback; this
+     * test is to check that there is no such crash now.  Instead, librsvg
+     * will return a 1x1 transparent pixbuf.
+     */
+    char *filename = get_test_filename ("example.svg");
+    GError *error = NULL;
+    RsvgHandle *handle;
+    GdkPixbuf *pixbuf;
+
+    handle = rsvg_handle_new_from_file (filename, &error);
+    g_free (filename);
+
+    g_assert (handle != NULL);
+    g_assert (error == NULL);
+
+    rsvg_handle_set_size_callback (handle, zero_size_func, NULL, NULL);
+
+    pixbuf = rsvg_handle_get_pixbuf (handle);
+    g_assert (pixbuf != NULL);
+    g_assert (gdk_pixbuf_get_width (pixbuf) == 1);
+    g_assert (gdk_pixbuf_get_height (pixbuf) == 1);
+
+    g_object_unref (pixbuf);
+    g_object_unref (handle);
+}
+
+static void
+pixbuf_size_func (gint *width, gint *height, gpointer user_data)
+{
+    *width = 420;
+    *height = 430;
+}
+
+static void
+get_pixbuf_with_size_callback (void)
+{
+    RsvgHandle *handle = rsvg_handle_new ();
+
+    rsvg_handle_set_size_callback (handle, pixbuf_size_func, NULL, NULL);
+
+    char *filename = get_test_filename ("example.svg");
+    guchar *data;
+    gsize length;
+    GError *error = NULL;
+
+    g_assert (g_file_get_contents (filename, (gchar **) &data, &length, &error));
+    g_assert (data != NULL);
+
+    g_free (filename);
+
+    g_assert (rsvg_handle_write (handle, data, length, &error));
+    g_assert (error == NULL);
+
+    g_assert (rsvg_handle_close (handle, &error));
+    g_assert (error == NULL);
+
+    GdkPixbuf *pixbuf = rsvg_handle_get_pixbuf (handle);
+    g_assert (pixbuf != NULL);
+    g_assert_cmpint (gdk_pixbuf_get_width (pixbuf), ==, 420);
+    g_assert_cmpint (gdk_pixbuf_get_height (pixbuf), ==, 430);
+
+    g_object_unref (pixbuf);
+    g_free (data);
+    g_object_unref (handle);
+}
+
 static void
 detects_cairo_context_in_error (void)
 {
@@ -825,6 +905,84 @@ render_cairo_sub (void)
     cairo_surface_destroy (surf);
 }
 
+static void
+get_intrinsic_dimensions (void)
+{
+    char *filename = get_test_filename ("example.svg");
+    GError *error = NULL;
+
+    RsvgHandle *handle = rsvg_handle_new_from_file (filename, &error);
+    g_free (filename);
+
+    g_assert (handle != NULL);
+    g_assert (error == NULL);
+
+    gboolean has_width;
+    RsvgLength width;
+    gboolean has_height;
+    RsvgLength height;
+    gboolean has_viewbox;
+    RsvgRectangle viewbox;
+
+    rsvg_handle_get_intrinsic_dimensions (handle, &has_width, &width, &has_height, &height, &has_viewbox, &viewbox);
+
+    g_assert (has_width);
+    g_assert_cmpfloat (width.length, ==, 100.0);
+    g_assert (width.unit == RSVG_UNIT_PX);
+
+    g_assert (has_height);
+    g_assert_cmpfloat (height.length, ==, 400.0);
+    g_assert (height.unit == RSVG_UNIT_PX);
+
+    g_assert (has_viewbox);
+    g_assert_cmpfloat (viewbox.x, ==, 0.0);
+    g_assert_cmpfloat (viewbox.y, ==, 0.0);
+    g_assert_cmpfloat (viewbox.width, ==, 100.0);
+    g_assert_cmpfloat (viewbox.height, ==, 400.0);
+
+    g_object_unref (handle);
+}
+
+static void
+get_geometry_for_element (void)
+{
+    char *filename = get_test_filename ("geometry.svg");
+    GError *error = NULL;
+
+    RsvgHandle *handle = rsvg_handle_new_from_file (filename, &error);
+    g_free (filename);
+
+    g_assert (handle != NULL);
+    g_assert (error == NULL);
+
+    RsvgRectangle viewport = { 0.0, 0.0, 100.0, 400.0 };
+    RsvgRectangle ink_rect;
+    RsvgRectangle logical_rect;
+
+    g_assert (!rsvg_handle_get_geometry_for_element (handle, "#nonexistent", &viewport,
+                                                     &ink_rect, &logical_rect, &error));
+    g_assert (error != NULL);
+
+    g_error_free (error);
+    error = NULL;
+
+    g_assert (rsvg_handle_get_geometry_for_element (handle, "#two", &viewport,
+                                                    &ink_rect, &logical_rect, &error));
+    g_assert (error == NULL);
+
+    g_assert_cmpfloat (ink_rect.x, ==, 5.0);
+    g_assert_cmpfloat (ink_rect.y, ==, 195.0);
+    g_assert_cmpfloat (ink_rect.width, ==, 90.0);
+    g_assert_cmpfloat (ink_rect.height, ==, 110.0);
+
+    g_assert_cmpfloat (logical_rect.x, ==, 10.0);
+    g_assert_cmpfloat (logical_rect.y, ==, 200.0);
+    g_assert_cmpfloat (logical_rect.width, ==, 80.0);
+    g_assert_cmpfloat (logical_rect.height, ==, 100.0);
+
+    g_object_unref (handle);
+}
+
 /* https://gitlab.gnome.org/GNOME/librsvg/issues/385 */
 static void
 no_write_before_close (void)
@@ -888,6 +1046,129 @@ cannot_request_external_elements (void)
     g_test_trap_assert_stderr ("*WARNING*the public API is not allowed to look up external references*");
 }
 
+static void
+test_flags (RsvgHandleFlags flags)
+{
+    guint read_flags;
+
+    RsvgHandle *handle = g_object_new (RSVG_TYPE_HANDLE,
+                                       "flags", flags,
+                                       NULL);
+    g_object_get (handle, "flags", &read_flags, NULL);
+    g_assert (read_flags == flags);
+
+    g_object_unref (handle);
+}
+
+static void
+property_flags (void)
+{
+    test_flags (RSVG_HANDLE_FLAGS_NONE);
+    test_flags (RSVG_HANDLE_FLAG_UNLIMITED);
+    test_flags (RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA);
+    test_flags (RSVG_HANDLE_FLAG_UNLIMITED | RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA);
+}
+
+static void
+property_dpi (void)
+{
+    RsvgHandle *handle = g_object_new (RSVG_TYPE_HANDLE,
+                                       "dpi-x", 42.0,
+                                       "dpi-y", 43.0,
+                                       NULL);
+    double x, y;
+
+    g_object_get (handle,
+                  "dpi-x", &x,
+                  "dpi-y", &y,
+                  NULL);
+
+    g_assert (x == 42.0);
+    g_assert (y == 43.0);
+
+    g_object_unref (handle);
+}
+
+static void
+property_base_uri (void)
+{
+    RsvgHandle *handle = g_object_new (RSVG_TYPE_HANDLE,
+                                       "base-uri", "file:///foo/bar.svg",
+                                       NULL);
+    char *uri;
+
+    g_object_get (handle,
+                  "base-uri", &uri,
+                  NULL);
+
+    g_assert (strcmp (uri, "file:///foo/bar.svg") == 0);
+    g_free (uri);
+
+    g_object_unref (handle);
+}
+
+static void
+property_dimensions (void)
+{
+    char *filename = get_test_filename ("example.svg");
+    GError *error = NULL;
+
+    RsvgHandle *handle = rsvg_handle_new_from_file (filename, &error);
+    g_free (filename);
+
+    g_assert (handle != NULL);
+    g_assert (error == NULL);
+
+    int width;
+    int height;
+    double em;
+    double ex;
+
+    g_object_get (handle,
+                  "width", &width,
+                  "height", &height,
+                  "em", &em,
+                  "ex", &ex,
+                  NULL);
+
+    g_assert_cmpint (width,  ==, EXAMPLE_WIDTH);
+    g_assert_cmpint (height, ==, EXAMPLE_HEIGHT);
+
+    g_assert_cmpfloat (em, ==, (double) EXAMPLE_WIDTH);
+    g_assert_cmpfloat (ex, ==, (double) EXAMPLE_HEIGHT);
+
+    g_object_unref (handle);
+}
+
+static void
+property_deprecated (void)
+{
+    char *filename = get_test_filename ("example.svg");
+    GError *error = NULL;
+
+    RsvgHandle *handle = rsvg_handle_new_from_file (filename, &error);
+    g_free (filename);
+
+    g_assert (handle != NULL);
+    g_assert (error == NULL);
+
+    char *title;
+    char *desc;
+    char *metadata;
+
+    g_object_get (handle,
+                  "title", &title,
+                  "desc", &desc,
+                  "metadata", &metadata,
+                  NULL);
+
+    g_assert (title == NULL);
+    g_assert (desc == NULL);
+    g_assert (metadata == NULL);
+
+    g_object_unref (handle);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -918,12 +1199,21 @@ main (int argc, char **argv)
     g_test_add_func ("/api/dimensions_and_position", dimensions_and_position);
     g_test_add_func ("/api/set_size_callback", set_size_callback);
     g_test_add_func ("/api/reset_size_callback", reset_size_callback);
+    g_test_add_func ("/api/render_with_zero_size_callback", render_with_zero_size_callback);
+    g_test_add_func ("/api/get_pixbuf_with_size_callback", get_pixbuf_with_size_callback);
     g_test_add_func ("/api/detects_cairo_context_in_error", detects_cairo_context_in_error);
     g_test_add_func ("/api/can_draw_to_non_image_surface", can_draw_to_non_image_surface);
     g_test_add_func ("/api/render_cairo_sub", render_cairo_sub);
+    g_test_add_func ("/api/get_intrinsic_dimensions", get_intrinsic_dimensions);
+    g_test_add_func ("/api/get_geometry_for_element", get_geometry_for_element);
     g_test_add_func ("/api/no_write_before_close", no_write_before_close);
     g_test_add_func ("/api/empty_write_close", empty_write_close);
     g_test_add_func ("/api/cannot_request_external_elements", cannot_request_external_elements);
+    g_test_add_func ("/api/property_flags", property_flags);
+    g_test_add_func ("/api/property_dpi", property_dpi);
+    g_test_add_func ("/api/property_base_uri", property_base_uri);
+    g_test_add_func ("/api/property_dimensions", property_dimensions);
+    g_test_add_func ("/api/property_deprecated", property_deprecated);
 
     return g_test_run ();
 }
