@@ -17,11 +17,8 @@ typedef struct
     gdouble y;
     gdouble width;
     gdouble height;
-    gdouble logical_width;
-    gdouble logical_height;
     gboolean has_position;
     gboolean has_dimensions;
-    gboolean has_logical_dimensions;
 } FixtureData;
 
 /* The following are stolen from g_assert_cmpfloat_with_epsilon() and
@@ -47,8 +44,6 @@ test_dimensions (FixtureData *fixture)
     RsvgHandle *handle;
     RsvgPositionData position;
     RsvgDimensionData dimension;
-    RsvgRectangle ink_rect;
-    RsvgRectangle logical_rect;
     gchar *target_file;
     GError *error = NULL;
 
@@ -62,33 +57,20 @@ test_dimensions (FixtureData *fixture)
         g_assert (rsvg_handle_has_sub (handle, fixture->id));
         g_assert (rsvg_handle_get_position_sub (handle, &position, fixture->id));
         g_assert (rsvg_handle_get_dimensions_sub (handle, &dimension, fixture->id));
-        g_assert (rsvg_handle_get_geometry_sub (handle, &ink_rect, &logical_rect, fixture->id));
 
         g_message ("w=%d h=%d", dimension.width, dimension.height);
     } else {
         rsvg_handle_get_dimensions (handle, &dimension);
-        rsvg_handle_get_geometry_sub (handle, &ink_rect, &logical_rect, NULL);
     }
 
     if (fixture->has_position) {
         g_assert_cmpint (fixture->x, ==, position.x);
         g_assert_cmpint (fixture->y, ==, position.y);
-
-        my_assert_cmpfloat_with_epsilon (fixture->x, ink_rect.x, 0.01);
-        my_assert_cmpfloat_with_epsilon (fixture->y, ink_rect.y, 0.01);
     }
 
     if (fixture->has_dimensions) {
         g_assert_cmpint (fixture->width,  ==, dimension.width);
         g_assert_cmpint (fixture->height, ==, dimension.height);
-
-        my_assert_cmpfloat_with_epsilon (fixture->width,  ink_rect.width, 0.01);
-        my_assert_cmpfloat_with_epsilon (fixture->height, ink_rect.height, 0.01);
-    }
-
-    if (fixture->has_logical_dimensions) {
-        my_assert_cmpfloat_with_epsilon (fixture->logical_width,  logical_rect.width, 0.01);
-        my_assert_cmpfloat_with_epsilon (fixture->logical_height, logical_rect.height, 0.01);
     }
 
     g_object_unref (handle);
@@ -100,47 +82,37 @@ static FixtureData fixtures[] =
         "/dimensions/no viewbox, width and height",
         "dimensions/bug608102.svg",
         NULL,
-        0, 0, 16, 16, 16, 16,
-        FALSE, TRUE, TRUE
+        0, 0, 16, 16,
+        FALSE, TRUE
     },
     {
         "/dimensions/100% width and height",
         "dimensions/bug612951.svg",
         NULL,
-        0, 0, 47, 47.14, 44.546, 45.44,
-        FALSE, TRUE, TRUE
+        0, 0, 47, 47.14,
+        FALSE, TRUE
     },
     {
         "/dimensions/viewbox only",
         "dimensions/bug614018.svg",
         NULL,
-        0, 0, 972, 546, 0, 0,
+        0, 0, 972, 546,
         FALSE, TRUE
     },
     {
         "/dimensions/sub/rect no unit",
         "dimensions/sub-rect-no-unit.svg",
         "#rect-no-unit",
-        0, 0, 44, 45, 0, 0,
+        0, 0, 44, 45,
         FALSE, TRUE
     },
     {
         "/dimensions/sub/text_position",
         "dimensions/347-wrapper.svg",
         "#LabelA",
-        80, 48.90, 0, 0, 0, 0,
+        80, 48.90, 0, 0,
         TRUE, FALSE
     },
-    /* FIXME: The wilber test fails, the ink_rect has the wrong size */
-    /*{
-        "/dimensions/sub/bug760112-wilber",
-        "dimensions/bug760112-wilber.svg",
-        "#g39819",
-        0, 0, 16, 16, 16, 16,
-        FALSE, TRUE, TRUE
-    },
-    */
-    /* {"/dimensions/sub/rect with transform", "dimensions/bug564527.svg", "#back", 0, 0, 144, 203} */
 };
 
 static const gint n_fixtures = G_N_ELEMENTS (fixtures);
@@ -153,6 +125,8 @@ main (int argc, char *argv[])
 
     g_test_init (&argc, &argv, NULL);
 
+    test_utils_setup_font_map ();
+
     for (i = 0; i < n_fixtures; i++)
         g_test_add_data_func (fixtures[i].test_name, &fixtures[i], (void*)test_dimensions);
 
